/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.layers;

import com.vividsolutions.jts.geom.Geometry;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.beans.IntrospectionException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerObject;
import nl.cloudfarming.client.geoviewer.MapController;
import nl.cloudfarming.client.model.Bean;
import org.openide.explorer.propertysheet.PropertyPanel;
import org.openide.nodes.BeanNode;
import org.openide.nodes.Children;
import org.openide.util.WeakListeners;
import org.openide.util.lookup.Lookups;
import org.openide.windows.TopComponent;

/**
 *
 * @author Timon Veenstra
 */
public class LayerObjectNode<G extends Geometry, B extends Bean, L extends LayerObject<G, B>> extends BeanNode<B> implements PropertyChangeListener {

    private PropertyPanel itemPP;
    private TopComponent tc = new TopComponent();
    private final L layerObject;
    private final Layer<L> layer;

    public LayerObjectNode(Layer<L> layer, L layerObject) throws IntrospectionException {
        super(layerObject.getObject(), Children.LEAF, Lookups.fixed(layer, layerObject));
        this.layerObject = layerObject;
        this.layer = layer;
        layerObject.getObject().addPropertyChangeListener(WeakListeners.propertyChange(this, layerObject.getObject()));
    }

    public L getLayerObject() {
        return this.layerObject;
    }

    /**
     * Returns the HTML display name based on the visibility of the layer.
     *
     * @return The HTML display name of the node (layer)
     */
    @Override
    public String getHtmlDisplayName() {
        String displayName = layerObject.getTooltipText();

        return "<font color='#000000'>" + displayName + "</font>";
    }

    @Override
    public Action[] getActions(boolean context) {
        return new Action[]{new OpenDetailsAction(), new ZoomToAction()};
    }

    @Override
    public Action getPreferredAction() {
        return new ZoomToAction();
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if ("name".equals(evt.getPropertyName())) {
            this.fireDisplayNameChange(null, getDisplayName());
        }
    }

    public class ZoomToAction extends AbstractAction {

        public ZoomToAction() {
            //FIXME get name from bundle 
            super("zoom", null);
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            MapController controller = MapController.Registry.getLookup().lookup(MapController.class);
            controller.zoomToObject(getLayerObject(), layer);
            firePropertyChange(NAME, layer, enabled);
            setValue("zoom", true);
        }
    }

    public class OpenDetailsAction extends AbstractAction {

        public OpenDetailsAction() {
            //FIXME get name from bundle
            super("edit", null);
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            JPanel panel = new JPanel(new MigLayout());
            tc.removeAll();
            tc.setDisplayName(getDisplayName());
            tc.setLayout(new BorderLayout());
            panel.setBorder(new LineBorder(Color.RED, 2));
            tc.add(panel, BorderLayout.CENTER);
            PropertySet[] sets = getPropertySets();
            for (Property property : sets[0].getProperties()) {
                if (!property.getName().equals("class")) {
                    itemPP = new PropertyPanel();
                    itemPP.setProperty(property);
                    panel.add(new JLabel(property.getName().toUpperCase() + ": "));
                    panel.add(itemPP, "wrap, gaptop 10");
                    tc.repaint();
                    tc.validate();
                }
            }
            tc.open();
            tc.requestActive();
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final LayerObjectNode other = (LayerObjectNode) obj;
        if (this.layerObject != other.layerObject && (this.layerObject == null || !this.layerObject.equals(other.layerObject))) {
            return false;
        }
        if (this.layer != other.layer && (this.layer == null || !this.layer.equals(other.layer))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 23 * hash + (this.layerObject != null ? this.layerObject.hashCode() : 0);
        hash = 23 * hash + (this.layer != null ? this.layer.hashCode() : 0);
        return hash;
    }
//    @Override
//    protected SystemAction[] createActions() {
//
//        return new SystemAction[]{SystemAction.get(SelectInMapAction.class)};
//    }
//
//
//
//    @Override
//    public SystemAction getDefaultAction() {
//        return SystemAction.get(SelectInMapAction.class);
//    }
}
